/*
 * Linball - un pinball 2D multiplataforma escrito en C + SDL (desarrollado en Linux)
 *
 *
 * Copyright (c) 2007-2008 by Rafael Ignacio Zurita <rizurita@yahoo.com>
 * Copyright (c) 2007-2008 by Miriam Lechner <lechnermiriam@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>

#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL/SDL_ttf.h>
//#include <SDL_image.h>
#include "inicio_menu.c"
#include "eventos.c"

/*
 * dibujar_fondo_sup es el puente que está arriba de la tabla y que la pelota
 * puede llegar a alcanzar de acuerdo a ciertas condiciones
 */
void dibujar_fondo_sup(void)
{
	SDL_Rect dest;
	dest.x=layout[fondo_sup]->posx;
	dest.y = layout[fondo_sup]->posy+y0pantalla;
	dest.w = layout[fondo_sup]->imagen->w;
	dest.h = layout[fondo_sup]->imagen->h;
	SDL_BlitSurface(layout[fondo_sup]->imagen,NULL,scr,&dest);

};

void dibujar_fondo(void){
 const int borde=200 *MODY;
 int inc;
 /*Umbrales que delimitan el scroll*/
 int umbral_sup, umbral_inf; 

 /*incremento en y en valor absoluto*/
 inc = dy < 0 ? -dy : dy;

 umbral_sup=borde;
 umbral_inf=LARGO-borde;

 #ifdef DEBUG1
 printf("Delta y %d\n",inc);
 #endif

 //Definimos donde queremos dibujar el grafico
 //y lo copiamos a la pantalla
 dest.x=0;
 dest.w = layout[fondo]->imagen->w;
 dest.h = layout[fondo]->imagen->h;

 yp=pelotay+y0pantalla;
	

 if ((yp>umbral_inf)&&(y0pantalla>((dest.h*(-1))+LARGO))){
 #ifdef DEBUG1
 printf("yp es mayor que el umbral inferior\n");
 #endif
 
 y0pantalla=y0pantalla-inc;
 
 if (y0pantalla<(-dest.h+LARGO)){
       y0pantalla=-dest.h+LARGO;
    }
 }
 else if (((yp<=umbral_sup)&&(y0pantalla<0))){
 #ifdef DEBUG1
 printf("yp es menor  que el umbral superior\n");
 #endif
 
    y0pantalla=y0pantalla+inc;
    if (y0pantalla>0) { y0pantalla=0;}	
 }

 dest.y = y0pantalla;
 yp=y0pantalla+pelotay;

 #ifdef DEBUG1
 printf("pelotay yp y0pantalla %d %d %d\n",pelotay,yp,y0pantalla);
 #endif
 SDL_BlitSurface(layout[fondo]->imagen,NULL,scr,&dest);

 /*IMAGEN DE FONDO DUMMY*/
 SDL_BlitSurface(dummy,NULL,layout[fondo_dummy]->imagen,NULL);

 if (layout[fondo_dummy]->imagen == NULL){
       printf("No se pudo convertir el grafico %s\n",SDL_GetError());
       exit(1);
 }

}	


int colisiones()
{
       /* 
         * el color rojo es el color color_clave=63488; 
         * el color azul es el color color_clave=31; 
         * Ojo al piojo, este color clave rojo es en 16bit mode (cuando se inicializa
         * la pantalla se debe colocar como profundidad de colores 16
         */
        int color_clave=63488;

        /* el ancho (px) y el alto (py) en pixels de la bola */
        int px = 32 *MOD_BOLA;
        int py = 32 *MOD_BOLA;

        /* maxX es la resolucion en ancho de la pantalla, por ej. 640 pixels */
        int maxX = ANCHO;

        /* si efecto = 1 luego de los if se reproduce un sonido porque hubo una colision */
        int efecto = 1;


        /* 
         * posiciones a chequear si hay impacto (o colision) :
         *                    1
         *               5        6
         *           
         *           3                4
         *
         *               7        8
         *                    2
         */
        int pos1x = pelotax + (px/2);
        int pos1y = pelotay;
        int pos1 = maxX * pos1y + pos1x;

        int pos2x = pelotax + (px/2);
        int pos2y = pelotay + py;
        int pos2 = maxX * pos2y + pos2x;

        int pos3x = pelotax;
        int pos3y = pelotay + (py/2);
        int pos3 = maxX * pos3y + pos3x;

        int pos4x = pelotax + px;
        int pos4y = pelotay + (py/2);
        int pos4 = maxX * pos4y + pos4x;

        int pos5x = pelotax + (px/4) - 4;
        int pos5y = pelotay + (py/4) - 4;
        int pos5 = maxX * pos5y + pos5x;

        int pos6x = pelotax + px - (px/4) + 4;
        int pos6y = pelotay + (py/4) - 4;
        int pos6 = maxX * pos6y + pos6x;

        int pos7x = pelotax + (px/4) - 4;
        int pos7y = pelotay + py - (py/4) + 4;
        int pos7 = maxX * pos7y + pos7x;

        int pos8x = pelotax + px - (px/4) + 4;
        int pos8y = pelotay + py - (py/4) + 4;
        int pos8 = maxX * pos8y + pos8x;

        /* 
         * pix es un puntero al primer pixel de la pantalla (scr).
         * Cuando uno quiere chequear el pixel 2 (x=2) de la segunda fila (y=2)
         * tiene que chequear el pixel pix[maxX * y + x]
         */
        Uint16 * pix;
	if (puente) {
		pix = (Uint16 *) layout[fondo_sup_dummy]->imagen->pixels;
	} else {
		pix = (Uint16 *) layout[fondo_dummy]->imagen->pixels;
	};

        /*
         * el primer if es para que no se chequeen valores fuera de la pantalla
         * porque si se chequean valores fuera de la pantalla dará un
         * segmentation fault (porque no hay contra que chequear)
         */
        if (pelotay<10) {
                efecto = 0;

        } else if ((pix[pos8] == color_clave) && (dy>0) && (dx>0)) {
                dy = dy / -4;
                dx = dx * (-1) + dx/4;
                if (dx==0) dx=-2;
        } else if ((pix[pos8] == color_clave) && (dy>0) && (dx==0)) {
                dx = dy / -2;if (dx==0) dx=-2;
                dy = -1;
        } else if ((pix[pos8] == color_clave) && (dy>0) && (dx<0)&& (pix[pos7] != color_clave)) {
		if (dy>6) dy=dy/-4;
		else dy = dx / -4; if (dy<0) dy *= -1;
                if (dy==0) dy=1;
                if (dx > -2) dx=-2;
                dx = dx -1; if (dx<-6) dx=-6;
        } else if ((pix[pos8] == color_clave) && (pix[pos7] != color_clave)) {
                dy = dx / -4; if (dy<0) dy *= -1;
                if (dy==0) dy=1;
                if (dx > -2) dx=-2;
                dx = dx -1; if (dx<-6) dx=-6;

        } else if ((pix[pos7] == color_clave) && (dy>0) && (dx<0)) {
                dy = dy / -4;
                dx = dx * (-1) + dx/4;
                if (dx==0) dx=2;
        } else if ((pix[pos7] == color_clave) && (dy>0) && (dx==0)) {
                dx = dy / 2;if (dx==0) dx=2;
                dy = -1;
        } else if ((pix[pos7] == color_clave) && (dy>0) && (dx>0) && (pix[pos8] != color_clave)) {
		if (dy>6) dy=dy/-4;
		else dy = dx / -4; if (dy<0) dy *= -1;
                if (dy==0) dy=1;
                if (dx < 2) dx=2;
                dx = dx +1; if (dx>6) dx=6;
        } else if ((pix[pos7] == color_clave) && (pix[pos8] != color_clave)) {
                dy = dx / -4; if (dy<0) dy *= -1;
                if (dy==0) dy=1;
                if (dx < 2) dx=2;
                dx = dx +1; if (dx>6) dx=6;

        /*
         * si la pelota choca en su lado superior o inferior entonces solamente
         * la desviamos al lado opuesto
         */
        } else if (((pix[pos1] == color_clave) && (dy<0)) ||
                  ((pix[pos2] == color_clave) && (dy>0))) {
                if (dy>0) {
			dy = dy / -4;
			pelotay -= 1;
			if (dy==0) {dy=-1;/* pelotay -=1;*/};
		} else {
			dy = dy / -4;
			pelotay += 1;
			if (dy==0) {dy=1;/* pelotay -=1;*/};
		}
		
                if ((dx<0) && (dx>-3)) dx = dx -1; else if (dx<3) dx = dx + 1;

        } else if ((pix[pos5] == color_clave) && (dy<0) && (dx<0)) {
                dx = dy / 4; if (dx<0) dx *= (-1); else if (dx==0) dx=1;
                dy = dy / 4;
        } else if ((pix[pos5] == color_clave) && (dy<0) && (dx==0)) {
                dx = dy / 4; if (dx<0) dx *= (-1); else if (dx==0) dx=1;
                dy = dy / 4;
        } else if ((pix[pos5] == color_clave) && (dx>0) && (dy<0) && (pix[pos6] != color_clave)) {
                dx = dy / 4; if (dx<0) dx *= (-1); else if (dx==0) dx=1;
                dy = dy / 4;
        } else if (pix[pos5] == color_clave) {
                dx = dy / 4; if (dx<0) dx *= (-1); else if (dx==0) dx=1;
                dy = dy / -4;


	} else if ((pix[pos6] == color_clave) && (dy<0) && (dx>0)) {
                dx = dy / 4; if (dx>0) dx *= (-1); else if (dx==0) dx=-1;
                dy = dy / 4;
        } else if ((pix[pos6] == color_clave) && (dy<0) && (dx==0)) {
                dx = dy / 4; if (dx>0) dx *= (-1); else if (dx==0) dx=-1;
                dy = dy / 4;
        } else if ((pix[pos6] == color_clave) && (dx<0) && (dy<0) && (pix[pos5] != color_clave)) {
                dx = dy / 4; if (dx>0) dx *= (-1); else if (dx==0) dx=-1;
                dy = dy / 4;
        } else if (pix[pos6] == color_clave) {
                dx = dy / 4; if (dx>0) dx *= (-1); else if (dx==0) dx=-1;
                dy = dy / -4;



        /*
         * si la bola choca contra la izq o la der entonces solamente la desviamos
         * hacia al lado opuesto
         */
        } else if (((pix[pos3] == color_clave) && (dx<0)) ||
                  ((pix[pos4] == color_clave) && (dx>0))) {
                dx = dx * -1 / 2;


        } else efecto = 0;

        /* Reproducir un sonido si hubo una colision */
        if (efecto) {
		play_sound(layout[ball]->sonido, 128);
        };
	#ifdef DEBUG2
        printf("y=%d, x=%d, color_clave=%d \n",pelotay, pelotax, color_clave);
	#endif 

	return efecto;
}


int colision(Uint16 * pix, int xt, int yt, int color_clave)
{
	int ret = 0;	/* return temp */

        /* el ancho (px) y el alto (py) en pixels de la bola */
        int px = 32 *MOD_BOLA;
        int py = 32 *MOD_BOLA;

        /* maxX es la resolucion en ancho de la pantalla, por ej. 640 pixels */
        int maxX = ANCHO;

        /* 
         * posiciones a chequear si hay impacto (o colision) :
         *                    1
         *               5        6
         *           
         *           3                4
         *
         *               7        8
         *                    2
         */
        int pos1x = xt + (px/2);
        int pos1y = yt;
        int pos1 = maxX * pos1y + pos1x;

        int pos2x = xt + (px/2);
        int pos2y = yt + py;
        int pos2 = maxX * pos2y + pos2x;

        int pos3x = xt;
        int pos3y = yt + (py/2);
        int pos3 = maxX * pos3y + pos3x;

        int pos4x = xt + px;
        int pos4y = yt + (py/2);
        int pos4 = maxX * pos4y + pos4x;

        int pos5x = xt + (px/4) - 4;
        int pos5y = yt + (py/4) - 4;
        int pos5 = maxX * pos5y + pos5x;

        int pos6x = xt + px - (px/4) + 4;
        int pos6y = yt + (py/4) - 4;
        int pos6 = maxX * pos6y + pos6x;

        int pos7x = xt + (px/4) - 4;
        int pos7y = yt + py - (py/4) + 4;
        int pos7 = maxX * pos7y + pos7x;

        int pos8x = xt + px - (px/4) + 4;
        int pos8y = yt + py - (py/4) + 4;
        int pos8 = maxX * pos8y + pos8x;



        /* 
         * pix es un puntero al primer pixel de la pantalla (scr).
         * Cuando uno quiere chequear el pixel 2 (x=2) de la segunda fila (y=2)
         * tiene que chequear el pixel pix[maxX * y + x]
         */


        /*
         * el primer if es para que no se chequeen valores fuera de la pantalla
         * porque si se chequean valores fuera de la pantalla dará un
         * segmentation fault (porque no hay contra que chequear)
         */
        if (yt<10) {
                ret = 0;

        } else if ( ((pix[pos8] == color_clave) && (dy>0) && (dx>0)) ||
        	  ((pix[pos8] == color_clave) && (dy>0) && (dx==0)) ||
        	    ((pix[pos8] == color_clave) && (pix[pos7] != color_clave)) ||

		  ((pix[pos7] == color_clave) && (dy>0) && (dx<0)) ||
		  ((pix[pos7] == color_clave) && (dy>0) && (dx==0)) ||
		    ((pix[pos7] == color_clave) && (pix[pos8] != color_clave)) ||

		  ((pix[pos5] == color_clave) && (dy<0) && (dx<0)) ||
		  ((pix[pos5] == color_clave) && (dy<0) && (dx==0)) ||
		  (pix[pos5] == color_clave) ||

		  ((pix[pos6] == color_clave) && (dy<0) && (dx>0)) ||
		  ((pix[pos6] == color_clave) && (dy<0) && (dx==0)) ||
		  (pix[pos6] == color_clave) ||

        /*
         * si la bola choca contra la izq o la der entonces solamente la desviamos
         * hacia al lado opuesto
         */
		  (((pix[pos3] == color_clave) && (dx<0)) ||
                  ((pix[pos4] == color_clave) && (dx>0))) ||

        /*
         * si la pelota choca en su lado superior o inferior entonces solamente
         * la desviamos al lado opuesto
         */
		  (((pix[pos1] == color_clave) && (dy<0)) ||
                  ((pix[pos2] == color_clave) && (dy>0))) )
			ret = 1;

	else ret = 0;

	return ret;

}


int obtener_inc (int inc, int d)
{
	if (d < 0) inc = inc * (-1);
	return inc;
}


int obtener_inc_simple (int inc, int menor, int mayor)
{
	if (mayor != 0)
		return (inc * menor / mayor);
	else
		return 0;
}


void prox_pos_de_la_pelota() 
{
	int tx,ty,dxt,dyt;

	if ((pelotax < 0) || (pelotax > 800))
		dx = dx * (-1);

	dy++;

	if ((dy!=0) || (dx!=0)) {

	if (dx<0) tx=dx*(-1); else tx=dx;
	if (dy<0) ty=dy*(-1); else ty=dy;


	int done = 0;
	int inc = 0;
	while(!done) {
		inc++;
		if (tx > ty) {
			dxt = obtener_inc (inc, dx);
			dyt = obtener_inc_simple (inc, ty, tx);
			if (dy < 0) dyt = dyt * (-1);
		} else {
			dyt = obtener_inc (inc, dy);
			dxt = obtener_inc_simple (inc, tx, ty);
			if (dx < 0) dxt = dxt * (-1);
		}

        	Uint16 * pix;
		if (puente)
			pix = (Uint16 *) layout[fondo_sup_dummy]->imagen->pixels;
		else
			pix = (Uint16 *) layout[fondo_dummy]->imagen->pixels;

		if ((pelotay+dyt+32 *MOD_BOLA>(1195 *MODY_REAL)) ||
		    (colision(pix, (pelotax + dxt), (pelotay + dyt), 63488)) ||
		    ((tx > ty) && (inc == tx)) ||
		    ((ty >= tx) && (inc == ty)))
			done = 1;
	};

	if (puente){
	pelotax += 7;
	pelotay += 7;
	dy = dx = 7;
	}else {
	pelotax += dxt;
	pelotay += dyt;
	}

	
	};			

}


void dibujar_pelota() 
{
	SDL_Rect origen, destino;

	origen.x = 0;
	origen.y = 0;
	origen.w = layout[ball]->imagen->w;
	origen.h = layout[ball]->imagen->h;
	destino.w = layout[ball]->imagen->w;
	destino.h = layout[ball]->imagen->h;

	destino.x = pelotax;
	destino.y = yp;

	SDL_BlitSurface(layout[ball]->imagen, &origen, scr, &destino);
}


void dibujar_flippers() 
{
	SDL_Rect origen, destino, dest_dummy;
        #ifdef DEBUG3
	printf("Dentro de dibujar_flippers()\n");
	#endif

		origen.x = 0;
		origen.y = 0;
	if (layout[flipper_Ldwn]->visible)
	{
		origen.w = layout[flipper_LdwnR]->imagen->w;
		origen.h = layout[flipper_LdwnR]->imagen->h;

		destino.x = layout[flipper_LdwnR]->posx;
		destino.y = layout[flipper_LdwnR]->posy+y0pantalla;

		SDL_BlitSurface(layout[flipper_LdwnR]->imagen, &origen, scr, &destino);

		dest_dummy.x = layout[flipper_Ldwn]->posx;
		dest_dummy.y = layout[flipper_Ldwn]->posy; 
		SDL_BlitSurface(layout[flipper_Ldwn]->imagen, &origen, layout[fondo_dummy]->imagen, &dest_dummy);
	};
	if (layout[flipper_Rdwn]->visible)
	{
		origen.w = layout[flipper_RdwnR]->imagen->w;
		origen.h = layout[flipper_RdwnR]->imagen->h;

		destino.x = layout[flipper_RdwnR]->posx;;
		destino.y = layout[flipper_RdwnR]->posy+y0pantalla;

		SDL_BlitSurface(layout[flipper_RdwnR]->imagen, &origen, scr, &destino);
		dest_dummy.x = layout[flipper_Rdwn]->posx;
		dest_dummy.y = layout[flipper_Rdwn]->posy; 
		SDL_BlitSurface(layout[flipper_Rdwn]->imagen, &origen,layout[fondo_dummy]->imagen, &dest_dummy);
	};
	if (layout[flipper_Lup]->visible)
	{
		origen.w = layout[flipper_LupR]->imagen->w;
		origen.h = layout[flipper_LupR]->imagen->h;

		destino.x = layout[flipper_LupR]->posx;
		destino.y = layout[flipper_LupR]->posy+y0pantalla;

		SDL_BlitSurface(layout[flipper_LupR]->imagen, &origen, scr, &destino);
		dest_dummy.x = layout[flipper_Lup]->posx;
		dest_dummy.y = layout[flipper_Lup]->posy; 
		SDL_BlitSurface(layout[flipper_Lup]->imagen, &origen,layout[fondo_dummy]->imagen, &dest_dummy);
	};
	if (layout[flipper_Rup]->visible)
	{
		origen.w = layout[flipper_RupR]->imagen->w;
		origen.h = layout[flipper_RupR]->imagen->h;

		destino.x = layout[flipper_RupR]->posx;
		destino.y = layout[flipper_RupR]->posy+y0pantalla;

		SDL_BlitSurface(layout[flipper_RupR]->imagen, &origen, scr, &destino);
		dest_dummy.x = layout[flipper_Rup]->posx;
		dest_dummy.y = layout[flipper_Rup]->posy; 
		SDL_BlitSurface(layout[flipper_Rup]->imagen, &origen,layout[fondo_dummy]->imagen, &dest_dummy);
	};
}


void dibujar_texto(const char *texto)
{
	SDL_Color foregroundColor = { 255, 255, 0 };
	SDL_Color backgroundColor = { 0, 0, 0 };
	SDL_Surface* textSurface = TTF_RenderText_Shaded(font, texto,
	foregroundColor, backgroundColor); 
	// Pass zero for width and height to draw the whole surface
	SDL_Rect textLocation = { 0, 0, 0, 0 };
	SDL_Rect textFondo = { 0, 0, 800, 25};
	SDL_Rect textFondo2 = { 0, 25, 800, 1};

	/* Colocar el texto del puntaje y mensajes extras */
	SDL_FillRect(scr, &textFondo, SDL_MapRGB(scr->format, 0, 0, 0));
	SDL_FillRect(scr, &textFondo2, SDL_MapRGB(scr->format, 200, 200, 200));

	SDL_BlitSurface(textSurface, NULL, scr, &textLocation);

	/* Liberar ttf */
	SDL_FreeSurface(textSurface);

}


void sonido_start(){
	/* Actually loads up the music */
	music = Mix_LoadMUS("music.ogg");
	/* This begins playing the music - the first argument is a
	 *   pointer to Mix_Music structure, and the second is how many
	 *   times you want it to loop (use -1 for infinite, and 0 to
	 *   have it just play once) */
	Mix_VolumeMusic(20);
	Mix_PlayMusic(music, -1);

}

void sonido_stop(){
	/* Stop the music from playing */
	Mix_HaltMusic();

	/* Unload the music from memory, since we don't need it
	 * 	   anymore */
	Mix_FreeMusic(music);
	music = NULL;
}

void cargar_sonidos(){
printf("cargamos los sonidos\n");
	/* Pre-load sound effects */
	layout[ball]->sonido = Mix_LoadWAV("sonido.wav"); 
	Mix_VolumeChunk(layout[ball]->sonido, 64);

	layout[flipper_Rup]->sonido = Mix_LoadWAV("flip_up_snd.wav"); 
	Mix_VolumeChunk(layout[flipper_Rup]->sonido, 64);

	layout[flipper_Lup]->sonido = Mix_LoadWAV("flip_up_snd.wav"); 
	Mix_VolumeChunk(layout[flipper_Lup]->sonido, 64);

	layout[flipper_Rdwn]->sonido = Mix_LoadWAV("flip_down_snd.wav"); 
	Mix_VolumeChunk(layout[flipper_Rdwn]->sonido, 64);

	layout[flipper_Ldwn]->sonido = Mix_LoadWAV("flip_down_snd.wav"); 
	Mix_VolumeChunk(layout[flipper_Ldwn]->sonido, 64);

	bola_cae_snd = Mix_LoadWAV("bola_cae_snd.wav"); 
	Mix_VolumeChunk(bola_cae_snd, 128);

	bola_doingdown = Mix_LoadWAV("bola_doingdown.wav"); 
	Mix_VolumeChunk(bola_doingdown, 128);

	doingdown = Mix_LoadWAV("doingdown.wav"); 
	Mix_VolumeChunk(doingdown, 128);

	bola_start_snd = Mix_LoadWAV("bola_start_snd.wav"); 
	Mix_VolumeChunk(bola_start_snd, 128);

	disparador_snd = Mix_LoadWAV("disparador_snd.wav"); 
	Mix_VolumeChunk(disparador_snd, 128);

	disparador_atras = Mix_LoadWAV("disparador_atras.wav"); 
	Mix_VolumeChunk(disparador_atras, 128);

	layout[bumperi1]->sonido = Mix_LoadWAV("images/bumperi1.wav"); 
	Mix_VolumeChunk(layout[bumperi1]->sonido, 64);

	layout[bumperi2]->sonido = Mix_LoadWAV("images/bumperi2.wav"); 
	Mix_VolumeChunk(layout[bumperi2]->sonido, 64);

	layout[bumperi3]->sonido = Mix_LoadWAV("images/bumperi3.wav"); 
	Mix_VolumeChunk(layout[bumperi3]->sonido, 64);

	layout[fondo_objetos]->sonido = Mix_LoadWAV("images/bumperi2.wav"); 
	Mix_VolumeChunk(layout[fondo_objetos]->sonido, 64);

	layout[bumpizq]->sonido = Mix_LoadWAV("bumpizq.wav"); 
	Mix_VolumeChunk(layout[bumpizq]->sonido, 64);

	layout[bumpder]->sonido = Mix_LoadWAV("bumpder.wav"); 
	Mix_VolumeChunk(layout[bumpder]->sonido, 64);

	layout[tabla_luz1]->sonido = Mix_LoadWAV("tabla_luz1.wav"); 
	Mix_VolumeChunk(layout[tabla_luz1]->sonido, 64);

	layout[tabla_luz2]->sonido = Mix_LoadWAV("tabla_luz2.wav"); 
	Mix_VolumeChunk(layout[tabla_luz2]->sonido, 64);

	layout[tabla_luz3]->sonido = Mix_LoadWAV("tabla_luz3.wav"); 
	Mix_VolumeChunk(layout[tabla_luz3]->sonido, 64);

	puntaje_s = Mix_LoadWAV("turururi.wav"); 
	Mix_VolumeChunk(puntaje_s, 32);

}

void cargar_imagenes(){
     /*Cargamos graficos*/

     /*IMAGEN DE FONDO*/
     temp = SDL_LoadBMP("images/personajes/personajes11.bmp");
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
 /*
 ** hacemos que imagen sea el bitmap cargado en tmp pero con el formato
 ** de profundidad de colores del display (la pantalla). Esto es 
 ** para que SDL no tenga que transformar la imagen al formato correcto
 ** cada vez que se copie el fondo a la pantalla
 **/

     layout[fondo]->imagen = SDL_DisplayFormat(temp);
     if (layout[fondo]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[fondo]->posx=0;
     layout[fondo]->posy=0;
     layout[fondo]->visible=1;

	/* fondo para controlar las colisiones con objetos */
     temp = SDL_LoadBMP("pinball_objetos.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[fondo_objetos]->imagen = SDL_DisplayFormat(temp);
     if (layout[fondo_objetos]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[fondo_objetos]->posx=0;
     layout[fondo_objetos]->posy=0;
     layout[fondo_objetos]->visible=0;

     /* aqui bonus banner */
     temp = SDL_LoadBMP("aquibonus.bmp");
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }

     layout[aquibonus]->imagen = SDL_DisplayFormat(temp);
     if (layout[aquibonus]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[aquibonus]->posx=622 *MODX;
     layout[aquibonus]->posy=1113 *MODY_REAL;
     layout[aquibonus]->visible=1;
     /* hacemos invisible el color blanco */
     SDL_SetColorKey(layout[aquibonus]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[aquibonus]->imagen->format, 255, 255, 255));
     /* hacemos un poco transparente este "puente" superior (para que la bola pase por debajo) */
     SDL_SetAlpha (layout[aquibonus]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, 230);



     /*IMAGEN SUPERIOR e IMAGEN SUPERIOR DUMMY*/
     temp = SDL_LoadBMP("pinball5.bmp");
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }

     layout[fondo_sup]->imagen = SDL_DisplayFormat(temp);
     if (layout[fondo_sup]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[fondo_sup]->posx=89 *MODX;
     layout[fondo_sup]->posy=520 *MODY_REAL;
     layout[fondo_sup]->visible=1;
     /* hacemos invisible el color blanco */
     SDL_SetColorKey(layout[fondo_sup]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[fondo_sup]->imagen->format, 255, 255, 255));
     /* hacemos un poco transparente este "puente" superior (para que la bola pase por debajo) */
     SDL_SetAlpha (layout[fondo_sup]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, 180);

     temp = SDL_LoadBMP("pinball4.bmp");
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[fondo_sup_dummy]->imagen = SDL_DisplayFormat(temp);
     if (layout[fondo_sup_dummy]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);

     layout[fondo_sup_dummy]->posx=0;
     layout[fondo_sup_dummy]->posy=0;
     layout[fondo_sup_dummy]->visible=0;


     #ifdef DEBUG1
     printf("color_clave=%d \n",color_clave);
     #endif

     /*IMAGEN DE FONDO DUMMY*/
     temp = SDL_LoadBMP("pinball.bmp");
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }

     dummy = SDL_DisplayFormat(temp);
     if (dummy == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }

     layout[fondo_dummy]->imagen = SDL_DisplayFormat(temp);
     if (layout[fondo_dummy]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     } 

     SDL_FreeSurface(temp);
     layout[fondo_dummy]->posx=0;
     layout[fondo_dummy]->posy=0;
     layout[fondo_dummy]->visible=0;

     /*PELOTA*/
     temp = SDL_LoadBMP("pelota.bmp");
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[ball]->imagen = SDL_DisplayFormat(temp);
     SDL_FreeSurface(temp);
     if (layout[ball]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     } 

     layout[ball]->posx=340 *MODX;
     layout[ball]->posy=300 *MODY_REAL;
     layout[ball]->visible=1;
     SDL_SetColorKey(layout[ball]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[ball]->imagen->format, 255, 255, 255));
     pelotax = 743 *MODX;
     pelotay = 600 *MODY_REAL;


	/* algunas imagenes siguiente son para alpha-blending.
	 * El manual de SDL dice que SDL_DisplayFormat es util para
	 * convertir las imagenes al mismo formato de visualizacion,
	 * pero que para las imagenes que queremos utilizar para alpha-blending
	 * podria no ser una buena idea utilizar esa funcion (porque borra
	 * los canales utilizados para un fast blending).
	 */
     /* LOS 3 BUMPERS */
     temp = SDL_LoadBMP("images/bumperi1.bmp"); 
     layout[bumperi1]->imagen = SDL_DisplayFormat(temp);
     layout[bumperi1]->posx = 358 *MODX;
     layout[bumperi1]->posy = 422 *MODY_REAL;
     layout[bumperi1]->visible = 0;
     SDL_FreeSurface(temp);

     temp = SDL_LoadBMP("images/bumperi2.bmp"); 
     layout[bumperi2]->imagen = SDL_DisplayFormat(temp);
     layout[bumperi2]->posx = 253 *MODX;
     layout[bumperi2]->posy = 295 *MODY_REAL;
     layout[bumperi2]->visible = 0;
     SDL_FreeSurface(temp);

     temp = SDL_LoadBMP("images/bumperi3.bmp"); 
     layout[bumperi3]->imagen = SDL_DisplayFormat(temp);
     layout[bumperi3]->posx = 467 *MODX;
     layout[bumperi3]->posy = 298 *MODY_REAL;
     layout[bumperi3]->visible = 0;
     SDL_FreeSurface(temp);

     /* LOS 2 BUMPERS de abajo */
     temp = SDL_LoadBMP("bumpizq.bmp"); 
     layout[bumpizq]->imagen = SDL_DisplayFormat(temp);
     layout[bumpizq]->posx = 108 *MODX;
     layout[bumpizq]->posy = 805 *MODY_REAL;
     layout[bumpizq]->visible = 0;
     SDL_SetColorKey(layout[bumpizq]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[bumpizq]->imagen->format, 255, 255, 255));
     SDL_FreeSurface(temp);

     temp = SDL_LoadBMP("bumpder.bmp"); 
     layout[bumpder]->imagen = SDL_DisplayFormat(temp);
     layout[bumpder]->posx = 409 *MODX;
     layout[bumpder]->posy = 812 *MODY_REAL;
     layout[bumpder]->visible = 0;
     SDL_SetColorKey(layout[bumpder]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[bumpder]->imagen->format, 255, 255, 255));
     SDL_FreeSurface(temp);

	/* fondos iluminado 1 2 y 3*/
	temp = SDL_LoadBMP("images/personajes/personajes12.bmp"); 
	if (temp == NULL){
		printf("No se pudo cargar grafico %s\n",SDL_GetError());
		exit(1);
	}
	layout[tabla_luz1]->imagen = SDL_DisplayFormat(temp);
	if (layout[tabla_luz1]->imagen == NULL){
		printf("No se pudo convertir el grafico %s\n",SDL_GetError());
		exit(1);
	}
	SDL_SetColorKey(layout[tabla_luz1]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[tabla_luz1]->imagen->format, 255, 255, 255));
	SDL_FreeSurface(temp);
	layout[tabla_luz1]->posx = 0 *MODX;
	layout[tabla_luz1]->posy = 0 *MODY_REAL;
	layout[tabla_luz1]->visible = 0;

	temp = SDL_LoadBMP("personajes14.bmp"); 
	if (temp == NULL){
		printf("No se pudo cargar grafico %s\n",SDL_GetError());
		exit(1);
	}
	layout[tabla_luz2]->imagen = SDL_DisplayFormat(temp);
	if (layout[tabla_luz2]->imagen == NULL){
		printf("No se pudo convertir el grafico %s\n",SDL_GetError());
		exit(1);
	}
	SDL_SetColorKey(layout[tabla_luz2]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[tabla_luz2]->imagen->format, 255, 255, 255));
	SDL_FreeSurface(temp);
	layout[tabla_luz2]->posx = 0 *MODX;
	layout[tabla_luz2]->posy = 380 *MODY_REAL;
	layout[tabla_luz2]->visible = 0;

	temp = SDL_LoadBMP("personajes15.bmp"); 
	if (temp == NULL){
		printf("No se pudo cargar grafico %s\n",SDL_GetError());
		exit(1);
	}
	layout[tabla_luz3]->imagen = SDL_DisplayFormat(temp);
	if (layout[tabla_luz3]->imagen == NULL){
		printf("No se pudo convertir el grafico %s\n",SDL_GetError());
		exit(1);
	}
	SDL_SetColorKey(layout[tabla_luz3]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[tabla_luz3]->imagen->format, 255, 255, 255));
	SDL_FreeSurface(temp);
	layout[tabla_luz3]->posx = 360 *MODX;
	layout[tabla_luz3]->posy = 0 *MODY_REAL;
	layout[tabla_luz3]->visible = 0;

     /* LAS 4 LUCES DE LA PASARELA */
     temp = SDL_LoadBMP("luz_verde.bmp"); 
     lucesd[verde]->imagen = SDL_DisplayFormat(temp);
     SDL_SetColorKey(lucesd[verde]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(lucesd[verde]->imagen->format, 255, 255, 255));
     SDL_FreeSurface(temp);

     temp = SDL_LoadBMP("luz_roja.bmp"); 
     lucesd[roja]->imagen = SDL_DisplayFormat(temp);
     SDL_SetColorKey(lucesd[roja]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(lucesd[roja]->imagen->format, 255, 255, 255));
     SDL_FreeSurface(temp);

     temp = SDL_LoadBMP("luz_azul.bmp"); 
     lucesd[azul]->imagen = SDL_DisplayFormat(temp);
     SDL_SetColorKey(lucesd[azul]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(lucesd[azul]->imagen->format, 255, 255, 255));
     SDL_FreeSurface(temp);

     temp = SDL_LoadBMP("luz_magenta.bmp"); 
     lucesd[magenta]->imagen = SDL_DisplayFormat(temp);
     SDL_SetColorKey(lucesd[magenta]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(lucesd[magenta]->imagen->format, 255, 255, 255));
     SDL_FreeSurface(temp);

	lucest[0]->posx = 300;
	lucest[0]->posy = 910;
	lucest[1]->posx = 460;
	lucest[1]->posy = 910;
	lucest[2]->posx = 250;
	lucest[2]->posy = 690;
	lucest[3]->posx = 500;
	lucest[3]->posy = 690;
	lucest[4]->posx = 390;
	lucest[4]->posy = 640;
	lucest[5]->posx = 120;
	lucest[5]->posy = 440;
	lucest[6]->posx = 160;
	lucest[6]->posy = 520;
	lucest[7]->posx = 200;
	lucest[7]->posy = 600;
	lucest[8]->posx = 670;
	lucest[8]->posy = 340;
	lucest[9]->posx = 630;
	lucest[9]->posy = 400;
	lucest[10]->posx = 590;
	lucest[10]->posy = 460;
	lucest[11]->posx = 550;
	lucest[11]->posy = 520;
	lucest[12]->posx = 220;
	lucest[12]->posy = 810;
	lucest[13]->posx = 530;
	lucest[13]->posy = 810;
	lucest[14]->posx = 320;
	lucest[14]->posy = 200;
	lucest[15]->posx = 390;
	lucest[15]->posy = 200;
	lucest[16]->posx = 460;
	lucest[16]->posy = 200;
	int i;
	for (i=0; i<17; i++){
		lucest[i]->clase = (rand()%4);
		lucest[i]->visible = (rand()%180)+20;
		lucest[i]->modo = (rand()%4);
		if (lucest[i]->modo == 0)
			lucest[i]->inc = 5; /* 5 : hace que vaya al ritmo de
					     * la musica :)
					     */
		else
			lucest[i]->inc = (rand()%3)+2;
	};


     /*FLIPPERS DOWN*/
     temp = SDL_LoadBMP("flipper_Ldwn.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_Ldwn]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_Ldwn]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_Ldwn]->posx=240 *MODX;
     layout[flipper_Ldwn]->posy=1110 *MODY_REAL;
     layout[flipper_Ldwn]->visible=1;
     SDL_SetColorKey(layout[flipper_Ldwn]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_Ldwn]->imagen->format, 255, 255, 255));

     temp = SDL_LoadBMP("flipper_Ldwn_real.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_LdwnR]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_LdwnR]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_LdwnR]->posx=240 *MODX;
     layout[flipper_LdwnR]->posy=1110 *MODY_REAL;
     layout[flipper_LdwnR]->visible=1;
     SDL_SetColorKey(layout[flipper_LdwnR]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_LdwnR]->imagen->format, 255, 255, 255));
     

     temp = SDL_LoadBMP("flipper_Rdwn.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_Rdwn]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_Rdwn]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_Rdwn]->posx=430 *MODX;
     layout[flipper_Rdwn]->posy=1110 *MODY_REAL;
     layout[flipper_Rdwn]->visible=1;
     SDL_SetColorKey(layout[flipper_Rdwn]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_Rdwn]->imagen->format, 255, 255, 255));

     temp = SDL_LoadBMP("flipper_Rdwn_real.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_RdwnR]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_RdwnR]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_RdwnR]->posx=430 *MODX;
     layout[flipper_RdwnR]->posy=1110 *MODY_REAL;
     layout[flipper_RdwnR]->visible=1;
     SDL_SetColorKey(layout[flipper_RdwnR]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_RdwnR]->imagen->format, 255, 255, 255));


     /*FLIPPERS UP*/
     temp = SDL_LoadBMP("flipper_Lup.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_Lup]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_Lup]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_Lup]->posx=240 *MODX;
     layout[flipper_Lup]->posy=1050 *MODY_REAL;
     layout[flipper_Lup]->visible=0;
     SDL_SetColorKey(layout[flipper_Lup]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_Lup]->imagen->format, 255, 255, 255));

     temp = SDL_LoadBMP("flipper_Lup_real.bmp"); 
     if (temp == NULL){
           printf("No se pudo cargar grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_LupR]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_LupR]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_LupR]->posx=240 *MODX;
     layout[flipper_LupR]->posy=1050 *MODY_REAL;
     layout[flipper_LupR]->visible=0;
     SDL_SetColorKey(layout[flipper_LupR]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_LupR]->imagen->format, 255, 255, 255));



     temp = SDL_LoadBMP("flipper_Rup.bmp"); 
     if (temp == NULL){
           printf("No se pudo carg35 grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_Rup]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_Rup]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_Rup]->posx=430 *MODX;
     layout[flipper_Rup]->posy=1050 *MODY_REAL;
     layout[flipper_Rup]->visible=0;
     SDL_SetColorKey(layout[flipper_Rup]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_Rup]->imagen->format, 255, 255, 255));

     temp = SDL_LoadBMP("flipper_Rup_real.bmp"); 
     if (temp == NULL){
           printf("No se pudo carg35 grafico %s\n",SDL_GetError());
           exit(1);
     }
     layout[flipper_RupR]->imagen = SDL_DisplayFormat(temp);
     if (layout[flipper_RupR]->imagen == NULL){
           printf("No se pudo convertir el grafico %s\n",SDL_GetError());
           exit(1);
     }
     SDL_FreeSurface(temp);
     layout[flipper_RupR]->posx=430 *MODX;
     layout[flipper_RupR]->posy=1050 *MODY_REAL;
     layout[flipper_RupR]->visible=0;
     SDL_SetColorKey(layout[flipper_RupR]->imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(layout[flipper_RupR]->imagen->format, 255, 255, 255));

}

void liberar_imagenes(){

    enum objects i;
    for (i=fondo ;i < last ;i++)
    SDL_FreeSurface(layout[i]->imagen);

}


void manejador_objetos()
{
	enum objects i;
	enum objects j=0;
	double d, distancia=1200;

    	for (i=bumperi1 ;i <= tabla_luz3 ;i++) {
			if ( (d=(sqrt( ( (pelotax-(layout[i]->posx+layout[i]->imagen->w/2)) *
					 (pelotax-(layout[i]->posx+layout[i]->imagen->w/2)) )+
			               ( (pelotay-(layout[i]->posy+layout[i]->imagen->h/2)) *
			                 (pelotay-(layout[i]->posy+layout[i]->imagen->h/2)) ) ) )) < distancia ) {
				distancia = d;
				j=i;
			};
	};

	if (distancia < 1200) {

		if ((j<tabla_luz1) && (layout[j]->visible<100)) {

			/* reproducir sonido del objeto */
                	play_sound(layout[j]->sonido, 128);
	

			if ((distancia < 150) && (j==bumpizq) && (!puente)) {
				if (dx<0) dx *= -1;
				dx+=3;
				if (dy>=0) dy*= -1; 
				dy-= (rand() % 20);// else dy+=4;
				puntaje+= 500;
			} else if ((distancia < 150) && (j==bumpder) && (!puente)) {
				if (dx>0) dx *= -1;
				dx-=3;
				if (dy>=0) dy*= -1; 
				dy-= (rand() % 20);// else dy+=4;
				puntaje+= 500;

			} else if ((distancia < 150) && (!puente) && (j<18)) {
				if (dx<0) dx-=1; else dx+=1;
				if (dy<0) dy-=(rand() % 30);// else dy+=4;
				puntaje+= 1000;
			};
		} else if (layout[j]->visible<30) {
			/* reproducir sonido del objeto */
                	play_sound(layout[j]->sonido, 128);
		};

		if (j<tabla_luz1)
			layout[j]->visible = 200;
		else
			layout[j]->visible = 100;
		/* falta hacer un sonido largo y luz de borde aleatoria */

	distancia=1200;
    	for (i=0 ;i < 17 ;i++) {
			if ( (d=(sqrt( ( (pelotax-(lucest[i]->posx+lucesd[lucest[i]->clase]->imagen->w/2)) *
					 (pelotax-(lucest[i]->posx+lucesd[lucest[i]->clase]->imagen->w/2)) )+
			               ( (pelotay-(lucest[i]->posy+lucesd[lucest[i]->clase]->imagen->h/2)) *
			                 (pelotay-(lucest[i]->posy+lucesd[lucest[i]->clase]->imagen->h/2)) ) ) )) < distancia ) {
				distancia = d;
				j=i;
			};
	};
	if ((distancia < 1200) && (lucest[j]->visible < 100)) {
		puntaje+= 100;
		if (lucest[j]->modo ==0) {
			lucest[j]->modo = (rand()%4);
			if (lucest[j]->modo == 0)
				lucest[j]->inc = 5; /* 5 : hace que vaya al ritmo de
						     * la musica :)
						     */
			else
				lucest[j]->inc = (rand()%3)+2;
		} else {
			lucest[j]->modo = 0;
			lucest[j]->visible = 180;
			lucest[j]->inc = 1;
		}
	}
		
	};

	/* hacer sonar un bumper aleatoriamente */
	int objeto=15;
	if ((rand() % 11)==10){
		objeto += (rand() % 4);
		/* reproducir sonido del objeto */
                play_sound(layout[objeto]->sonido, 128);
		layout[objeto]->visible = 200;
	}


};

void dibujar_aquibonus(){
	SDL_Rect dest;
	dest.x = layout[aquibonus]->posx;
	dest.y = layout[aquibonus]->posy + y0pantalla;
	dest.w = layout[aquibonus]->imagen->w;
	dest.h = layout[aquibonus]->imagen->h;
	SDL_BlitSurface(layout[aquibonus]->imagen, NULL, scr, &dest);

}

void dibujar_objetos()
{
	SDL_Rect dest;
	enum objects i;
	int transparencia;

    	for (i=bumperi1 ;i <= tabla_luz3; i++)
		if (layout[i]->visible > 10) {
			if (i<tabla_luz1)
				layout[i]->visible = layout[i]->visible - 10; 
			else
				layout[i]->visible = layout[i]->visible - 5; 
			dest.x = layout[i]->posx;
			dest.y = layout[i]->posy + y0pantalla;
			dest.w = layout[i]->imagen->w;
			dest.h = layout[i]->imagen->h;
			SDL_SetAlpha (layout[i]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, layout[i]->visible);
			SDL_BlitSurface(layout[i]->imagen, NULL, scr, &dest);
		}
	
    	for (i=0 ;i < 17 ;i++) {
			if (lucest[i]->visible > 20)
				lucest[i]->visible = lucest[i]->visible - lucest[i]->inc; 
			if (lucest[i]->visible <= 20)
				lucest[i]->visible = 130;

			dest.x = lucest[i]->posx;
			dest.y = lucest[i]->posy + y0pantalla;
			dest.w = lucesd[lucest[i]->clase]->imagen->w;
			dest.h = lucesd[lucest[i]->clase]->imagen->h;

			if (lucest[i]->modo==0)
				transparencia = lucest[i]->visible;
//				SDL_SetAlpha (lucesd[(lucest[i]->clase)]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, lucest[i]->visible);
			else if (lucest[i]->modo==1) 
				transparencia = 40;
//				SDL_SetAlpha (lucesd[(lucest[i]->clase)]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, 40);
			else if (lucest[i]->modo==2) {
				if (lucest[i]->visible <90)
					transparencia = 40;
//					SDL_SetAlpha (lucesd[(lucest[i]->clase)]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, 40);
				else
					transparencia = 100;
//					SDL_SetAlpha (lucesd[(lucest[i]->clase)]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, 100);
			} else
					transparencia = 140;
//				SDL_SetAlpha (lucesd[(lucest[i]->clase)]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, 140);

			SDL_SetAlpha (lucesd[(lucest[i]->clase)]->imagen, SDL_SRCALPHA|SDL_RLEACCEL, transparencia);
			SDL_BlitSurface(lucesd[(lucest[i]->clase)]->imagen, NULL, scr, &dest);
		}
};



void inicio_del_nivel ()
{
	potencia_disparador = 0;
	nivel = 1;
	bolas_por_jugar = 3;
	chance_izq = 1;
	chance_der = 1;
	puntaje = 0;
	puntaje_anterior = 0;
	tecla_space = 0;
	puente = 0;
	juego_finalizado = 0;
	bonus_timer = 0;

	/* pintamos de negro la pantalla y agregamos un cartel de ayuda */
        SDL_Rect fondo_negro = { 0 *MODX, 0 *MODY, 800 *MODX, 600 *MODY};
        SDL_FillRect(scr, &fondo_negro, SDL_MapRGB(scr->format, 0, 0, 0));
	cartel(font, "TECLAS  [ KEYS ]  :", "SPACE, S, L, ESC", 2000);

}

void ingresar_bola ()
{
	char *s;
	s = malloc(80);
	if (bolas_por_jugar == 1)
		sprintf ( s, "        ** BOLA %d INGRESANDO ... ** (ULTIMA)", bolas_por_jugar);
	else
		sprintf ( s, "        ** BOLA %d INGRESANDO ... **", bolas_por_jugar);

	dibujar_fondo();
	dibujar_flippers();	
	dibujar_objetos();
	dibujar_fondo_sup();
	dibujar_aquibonus();
	dibujar_texto(s);
	free(s);
		SDL_Flip(scr);

		/* bajo el volumen de la musica */
		if (enable_sound) Mix_VolumeMusic(5);

		/* reproducir sonido de la bola ingresando al tablero */
                play_sound(bola_cae_snd, 128);
		SDL_Delay(1500);

	pelotax = 743 *MODX;
	pelotay = 600 *MODY_REAL;

	/* subir el volumen de la musica */
	if (enable_sound) Mix_VolumeMusic(40);

	potencia_disparador = 0;
	bolas_por_jugar--;
	chance_izq = 1;
	chance_der = 1;
	tecla_space = 0;
}


/* 
 * chequear_puntaje: controla colisiones con colores especificos para saber que
 * puntaje dar
 */
void chequear_puntaje()
{
	/* ejemplo: si la pelota esta contra un color rojo sumamos uno al puntaje */
	//if (colision((pelotax), (pelotay), 63488)) puntaje++;

}


void bola_perdida ()
{
	dibujar_fondo ();
	dibujar_flippers();	
	dibujar_objetos();
	dibujar_fondo_sup();
	dibujar_aquibonus();

	/* texto que colocaremos en la parte superior de la pantalla */
	sprintf ( banner, "        ** OOUCCHHH!!    BOLA %d PERDIDA... **", bolas_por_jugar+1);
	dibujar_texto(banner);

	SDL_Flip(scr);

	/* bajo el volumen de la musica */
	if (enable_sound) Mix_VolumeMusic(5);

	/* reproducir sonido de la bola cayendo del tablero */
	play_sound(bola_doingdown, 128);

	/* reproducir sonido de lamento por haber perdido una bola (doingdown) */
	play_sound(doingdown, 128);
	SDL_Delay(2500);

	/* subir el volumen de la musica */
	if (enable_sound) Mix_VolumeMusic(40);

}


void espera (int tiempo_minimo) {
	/* delay de sincronizacion */
	prev_ticks = cur_ticks;
	cur_ticks = SDL_GetTicks();
	if ((cur_ticks-prev_ticks) <= tiempo_minimo) {
		SDL_Delay(tiempo_minimo - (cur_ticks-prev_ticks));
		//printf("ticks=%i dif=%i margen=%i, \n",cur_ticks, cur_ticks-prev_ticks, 30-(cur_ticks-prev_ticks));
	}

}


void game_over() {
//	char *banner_final = "                                 HTTP://LINBALL.SOURCEFORGE.NET, PROGRAMACION : RAFAEL IGNACIO ZURITA Y MIRIAM TAMARA LECHNER, SONIDO HTTP://FREESOUND.IUA.UPF.EDU/, ARTE REALIZADO CON GIMP ...                 ";
	char ban[80];
	int done = 0;
	int k=0; int i=0;
	SDL_Event evento;

    	for (i=0 ;i < 17 ;i++)
		{
			lucest[i]->visible = (rand() % 200);
			lucest[i]->modo = 0;
			lucest[i]->inc = (rand()%3)+2;
		}
	i=0;

	while (!done) {

	dibujar_fondo ();
	dibujar_flippers();	
	dibujar_objetos();
	dibujar_fondo_sup();
	dibujar_aquibonus();

//	sprintf(ban, "** JUEGO TERMINADO ** %i PUNTOS. BIEN HECHO.. ! ", puntaje, banner_final+i);
	sprintf(ban, " LINBALL -   ** JUEGO TERMINADO ** %i PUNTOS. BIEN HECHO.. ! ", puntaje);
	dibujar_texto(ban);

	SDL_Flip(scr);

//	if (k++ == 2) {
//		k=0;
//		i++;
//	}

//	if (i==strlen(banner_final)) i=0;

		while ( SDL_PollEvent(&evento) )
			if (evento.type == SDL_KEYDOWN )
				done=1;
	espera(25);
	}
}
	


void bonus_historico() {
	int fin=0;
	int i, j = (rand() % lastpersonaje), k=0;
	
	char *ban;
	ban = malloc(strlen(bonus_text[j]) + 80);

	if (enable_sound) Mix_VolumeMusic(100);

    	for (i=0 ;i < 17 ;i++)
		{
			lucest[i]->visible = (rand() % 200);
			lucest[i]->modo = 0;
			lucest[i]->inc = (rand()%3)+2;
		}

	while (strlen(bonus_text[j]+fin) > 0)  {
		if ((rand()%50) == 10) {
			play_sound(layout[(rand() % 5)+bumperi1]->sonido, 128);
    			for (i=0 ;i < 17 ;i++)
				lucest[i]->visible = 180;
		}

		if ((rand()%100) == 10) {
			play_sound(layout[tabla_luz1]->sonido, 128);
    			for (i=bumperi1 ;i <= tabla_luz3 ;i++)
				layout[i]->visible = 150;
				
		}



	dibujar_fondo ();
	dibujar_flippers();	
	dibujar_objetos();
	dibujar_fondo_sup();
	dibujar_aquibonus();

	sprintf(ban, "** B-O-N-U-S !! ** :: %s", bonus_text[j]+fin);
	dibujar_texto(ban);

	SDL_Flip(scr);

	if (k++ == 2) {
		k=0;
		fin++;
	}
	espera(25);
	}

	free(ban);

	bonus_timer = 500;

	dy=-60;
	dx=0;
	pelotay = 1100;
				
}


/* chequea si la bola salió de la tabla (el jugador pierde una bola) o si
 * la bola ingresó al "puente"
 */
void chequear_bola()
{
        Uint16 * pix;
	if (puente) {
		pix = (Uint16 *) layout[fondo_sup_dummy]->imagen->pixels;
	} else {
		pix = (Uint16 *) layout[fondo_dummy]->imagen->pixels;
	};

	if ((pelotay+32 *MOD_BOLA>(1195 *MODY_REAL)) && (pelotax>670))
			bonus_historico();

	else if ((pelotay+32 *MOD_BOLA>(1195 *MODY_REAL)) && (bolas_por_jugar==0)) {
		/* hacer lo que haya que hacer aca cuando el juego finaliza */
		bola_perdida ();
		game_over();
		juego_finalizado = 1;

	} else if (pelotay+32 *MOD_BOLA>(1195 *MODY_REAL)) {

			/* 
			 * hacer lo que haya que hacer aca cuando el jugador pierde una
			 * bola.
			 */
			bola_perdida ();
			ingresar_bola ();

	/* chequeamos si la bola sube al puente o no */
	} else if ((pelotay>528 *MODY_REAL) && (pelotay<980 *MODY_REAL) && (colision(pix,(pelotax), (pelotay), 31))) {
		if (puente == 0) {
			/* subimos al puente y subimos el volumen de la musica */
			if (enable_sound) Mix_VolumeMusic(80);
			puente = 1;
			puntaje+= 1000;
		} else {
			/* bajamos del puente y bajamos el volumen de la musica */
			if (enable_sound) Mix_VolumeMusic(40);
			puente = 0;
		}
	};

}




/* la siguiente funcion solo es necesaria para la version de Windows */
// #include <windows.h>
// #include <sys/time.h>
//
//   void __stdcall GetSystemTimeAsFileTime(FILETIME*);
//
//   void gettimeofday(struct timeval* p, void* tz /* IGNORED */)
//   {
//          union {
//             long long ns100; /*time since 1 Jan 1601 in 100ns units */
//                 FILETIME ft;
//          } now;
//
//      GetSystemTimeAsFileTime( &(now.ft) );
//      p->tv_usec=(long)((now.ns100 / 10LL) % 1000000LL );
//      p->tv_sec= (long)((now.ns100-(116444736000000000LL))/10000000LL);
//   }

int main(int argc, char * argv[])
{

  /* parametros del audio */
  int audio_rate = 22050;
  Uint16 audio_format = AUDIO_S16; /* 16-bit stereo */
  int audio_channels = 2;
  int audio_buffers = 1024;

  SDL_Event evento;
  int done = 0, main_done = 0, ult_col; /* ult_col se utiliza para memorizar si hubo colision o no */
  enum objects i;

  /* inicializamos el random */
  srand(time (0));

  atexit(SDL_Quit);

/* opciones */
static const char *optstring = "nfh";
 
  int opt = 0;
  int enable_fullscreen = 0;
        
  opt = getopt(argc, argv, optstring);
        
  while(opt != -1) 
    {
    switch( opt ) 
      {
      case 'n':
        enable_sound = 0;  
        break;                                
      case 'f':
        enable_fullscreen = 1;  
        break;
      case 'h':
        printf("usage: linball [-n] [-f] \n\t -n - nosound\n\t -f - fullscreen\n");
        exit(0);                                
      default: break;
      }                
      opt = getopt(argc, argv, optstring);
    }


	/* Init SDL: */
	if ((enable_sound) && (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0 )) {
		fprintf(stderr, "Can't init: %s\n", SDL_GetError());
		exit(1);
	} else if (SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Can't init: %s\n", SDL_GetError());
		exit(1);
	}

	/* Open scr: */
	if (enable_fullscreen)
		scr = SDL_SetVideoMode(ANCHO, LARGO, prof, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
	else
  		scr = SDL_SetVideoMode(ANCHO, LARGO, prof, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_SRCCOLORKEY|SDL_SRCALPHA|SDL_FULLSCREEN);
	if (scr == NULL){
		printf("No se pudo iniciar el modo grafico %s\n",SDL_GetError());
		exit(1);
	}

	/* Inicializar ttf */
	TTF_Init();
	font = TTF_OpenFont("arial.ttf", 24); // IMPORTANTE!: ver que tamaño para 320
	font_bonus = TTF_OpenFont("arial.ttf", 16); // IMPORTANTE!: ver que tamaño para 320
  
	/*Open audio*/
	if ((enable_sound) && (Mix_OpenAudio(audio_rate,audio_format,audio_channels,audio_buffers) < 0)) {
		fprintf(stderr, "Warning: Audio could not be setup for %i Hz 16-bit stereo.\nReason: %s\n", audio_rate, SDL_GetError());
		exit(1);
  	}

	/*Inicializa la estrucutra que representa los objetos que se redibujan en la pantalla*/
	for (i=fondo; i<last; i++) {
		layout[i]=(struct object *) malloc(sizeof(struct object));
		layout[i]->sonido = NULL;
	};

	/* Inicializa la estructura de luces */
	for (i=roja; i<ult_luz; i++) {
		lucesd[i]=(struct luces_colores *) malloc(sizeof(struct luces_colores));
		lucesd[i]->imagen = NULL;
		lucesd[i]->sonido = NULL;
	};
	/* Inicializa el arreglo con informacion de las luces en la tabla */
	for (i=0; i<20; i++) {
		lucest[i]=(struct luz *) malloc(sizeof(struct luz));
	};

	/* cargar imagenes y sonido */
	cargar_imagenes();
	if (enable_sound)
		cargar_sonidos();
	
	while (!main_done) {

		done = menues();	/* llamamos al menu principal */
  
		if (!done) {		/* iniciar juego */

			if (enable_sound)
				 sonido_start();

			inicio_del_nivel();
			ingresar_bola();

		} else main_done = 1; 	/* salir del juego */
  
	 
	cur_ticks = SDL_GetTicks();
  		while (!done) {

	dibujar_fondo ();
	dibujar_flippers();	
	chequear_bola();
	if (juego_finalizado) {done=1; break;};
	ult_col = colisiones();
	
	if  (colision((Uint16 *) layout[fondo_objetos]->imagen->pixels, pelotax, pelotay, 31)) 
		manejador_objetos();

	prox_pos_de_la_pelota();

	/* antigua posicion del while manejador de eventos */
	/* antigua posicion del "if tecla_space ..."  */
	
	dibujar_objetos();	

	if (puente) {
		dibujar_fondo_sup();
		dibujar_pelota();
	} else {
		dibujar_pelota();
		dibujar_fondo_sup();
	};


	/* chequea/modifica el puntaje */
	chequear_puntaje();


	dibujar_aquibonus();


     while(SDL_PollEvent(&evento))
     {
	   switch (evento.type){
		case SDL_QUIT: done = 1; break;
		case SDL_MOUSEBUTTONDOWN:
			manejador_mouse(evento.button.x, evento.button.y);
			printf("button %i at (%i,%i)\n", evento.button.button, evento.button.x, evento.button.y);
	      		break;
	      case SDL_KEYDOWN:
	      case SDL_KEYUP: 
                if (evento.key.keysym.sym == SDLK_ESCAPE)
			done = cartel(font, "FINALIZAR JUEGO? Y/N","[ EXIT GAME? Y/N ]",1);
		  else manejador_tecla(evento.key);break;
	   }
     }

	if (tecla_space) {
		potencia_disparador +=3;
		if (potencia_disparador > 110 *MODY_REAL) potencia_disparador = 110 *MODY_REAL;
	};


	if (bonus_timer != 0) {

		bonus_timer--;

		if ((puntaje - puntaje_anterior) != 0)
			sprintf (banner_b, "  BONUS X 2 = %i", (puntaje - puntaje_anterior)*2);

		if (bonus_timer == 0) {
			
			sprintf (banner_b, "  ");
			if (enable_sound) Mix_VolumeMusic(40);

			for (i=0; i<17; i++) {
				lucest[i]->visible = (rand()%180)+20;
				lucest[i]->modo = (rand()%4);
				if (lucest[i]->modo == 0)
					lucest[i]->inc = 5; /* 5 : hace que vaya al ritmo de
							     * la musica :)
							     */
				else
					lucest[i]->inc = (rand()%3)+2;
			};
		}
	}	

	if (puntaje != puntaje_anterior)
		play_sound(puntaje_s, 128);
	puntaje = puntaje + (puntaje - puntaje_anterior)*2;
	/* texto que colocaremos en la parte superior de la pantalla */
	sprintf ( banner, " LINBALL - BOLA %d  - %6i  -  %s", bolas_por_jugar+1, puntaje, banner_b);
	dibujar_texto(banner);
	puntaje_anterior = puntaje;

	SDL_Flip(scr);

	espera(30);

  }


	if (enable_sound)
		sonido_stop();
  	};

  liberar_imagenes();

	/* finalizar ttf */
	TTF_CloseFont(font);
	TTF_CloseFont(font_bonus);
	TTF_Quit();

printf("a partir de aca libero un monton de cosas \n");

	/*Libera cada sonido de cada elemento de la estrucutra*/
	if (enable_sound)
		for (i=fondo; i<last; i++)
			if (layout[i]->sonido != NULL)
				Mix_FreeChunk(layout[i]->sonido);
printf("libere soni\n");

	/*Libera el espacio de la estrucutra*/
	for (i=fondo; i<last; i++)
		free(layout[i]);

	/* Libera el espacio de la estructura de luces */
	for (i=roja; i<ult_luz; i++) {
		if (lucesd[i]->imagen != NULL)
			SDL_FreeSurface(lucesd[i]->imagen);
		if (lucesd[i]->sonido != NULL);
			Mix_FreeChunk(lucesd[i]->sonido);
		free(lucesd[i]);
		
	};
	/* Libera el espacio de el arreglo con informacion de las luces en la tabla */
	for (i=0; i<20; i++) {
		free(lucest[i]);
	};

printf("libere la estruct\n");

	/* Libero los sonidos */
	if (enable_sound) {
		Mix_FreeChunk(bola_cae_snd);
		Mix_FreeChunk(bola_doingdown);
		Mix_FreeChunk(doingdown);
		Mix_FreeChunk(bola_start_snd);
		Mix_FreeChunk(disparador_snd);
		Mix_FreeChunk(disparador_atras);
		Mix_FreeChunk(puntaje_s);
		free(music);
	}
printf("libere otros sonidos \n");

	SDL_FreeSurface(image);
	SDL_FreeSurface(scr);
	SDL_FreeSurface(pelota);
	//free(temp);
	//free(temp2);
	SDL_FreeSurface(dummy);


printf("libere otros cosas \n");

	/* Quit: */
	if (enable_sound)
		Mix_CloseAudio(); 
	SDL_Quit();
   
  exit(0);
}
